@@ -2,63 +2,53 @@ from nipype.pipeline import engine as pe
 from nipype.interfaces import utility as niu, afni
 
 from .hmc import init_bold_hmc_wf
-from .utils import init_bold_reference_wf
-from .resampling import init_bold_preproc_trans_wf, init_bold_commonspace_trans_wf
+from .bold_ref import init_bold_reference_wf
+from .resampling import init_bold_preproc_trans_wf
 from .stc import init_bold_stc_wf
-from .bias_correction import bias_correction_wf
-from .registration import init_bold_reg_wf
+from .inho_correction import init_inho_correction_wf
+from .registration import init_cross_modal_reg_wf
 from .confounds import init_bold_confs_wf
 from nipype.interfaces.utility import Function
 
 
-def init_bold_main_wf(opts, bias_cor_only=False, name='bold_main_wf'):
+def init_bold_main_wf(opts, inho_cor_only=False, name='bold_main_wf'):
     """
     This workflow controls the functional preprocessing stages of the pipeline when both
     functional and anatomical images are provided.
 
     **Parameters**
 
-        apply_despiking
-            whether to apply despiking using AFNI's 3dDespike https://afni.nimh.nih.gov/pub/dist/doc/program_help/3dDespike.html.
-        tr
-            repetition time for the EPI
-        tpattern
-            specification for the within TR slice acquisition method. The input is fed to AFNI's 3dTshift
-        no_STC
-            whether to apply slice timing correction (STC) or not
-        detect_dummy
-            whether to detect and remove dummy volumes at the beginning of the EPI Sequences
-        slice_mc
-            whether to apply slice-specific motion correction through 2D registration of each slice, which can improve the correction
-            of within-TR motion
-        coreg_script
-            path to registration script for EPI to anat coregistraion. The script must
-            follow the template structure of registration scripts in shell_scripts/.
-            Default is set to 'SyN' registration.
-        nativespace_resampling
-            Specified dimensions for the resampling of the corrected EPI in native space.
-        commonspace_resampling
-            Specified dimensions for the resampling of the corrected EPI in common space.
+        opts
+            parser options for preprocess
+        inho_cor_only
+            whether to run the bias correction steps, or further processing steps.
 
     **Inputs**
 
         bold
             Input BOLD series NIfTI file
-        anat_ref
-            Preprocessed anatomical image after bias field correction and denoising
-        anat_mask
-            Brain mask inherited from the common space registration
+        coreg_anat
+            Anatomical reference for BOLD alignment
+        coreg_mask
+            Brain mask for anatomical reference
         WM_mask
-            Eroded WM mask inherited from the common space registration
+            WM mask inherited from the common space registration
         CSF_mask
-            Eroded CSF mask inherited from the common space registration
+            CSF mask inherited from the common space registration
+        vascular_mask
+            vascular mask inherited from the common space registration
         labels
             Anatomical labels inherited from the common space registration
-        commonspace_transforms_list
-            list of transforms to be applied to resample to commonspace
-        commonspace_inverses
-            Specification for the application of inverse affine transform for
-            the provided commonspace transforms
+        unbiased_to_atlas_affine
+            affine transform from the dataset template space to the commonspace space
+        unbiased_to_atlas_warp
+            non-linear transform from the dataset template space to the commonspace space
+        native_to_unbiased_affine
+            affine transform from the subject anatomical to the dataset template space
+        native_to_unbiased_warp
+            non-linear transform from the subject anatomical to the dataset template space
+        commonspace_ref
+            commonspace anatomical template
 
     **Outputs**
 
@@ -68,14 +58,20 @@ def init_bold_main_wf(opts, bias_cor_only=False, name='bold_main_wf'):
             Initial EPI median volume subsequently used as 3D reference EPI volume
         motcorr_params
             motion parameters file provided from antsMotionCorr
+        init_denoise
+            Corrected 3D ref EPI after initial correction step
+        denoise_mask
+            resampled mask used for final denoising
         corrected_EPI
             3D reference EPI volume after bias field correction
-        itk_bold_to_anat
-            Composite transforms from the EPI space to the anatomical space
-        itk_anat_to_bold
-            Composite transforms from the anatomical space to the EPI space
         output_warped_bold
             Bias field corrected 3D EPI volume warped to the anatomical space
+        bold_to_anat_affine
+            affine transform from the EPI space to the anatomical space
+        bold_to_anat_warp
+            non-linear transform from the EPI space to the anatomical space
+        bold_to_anat_inverse_warp
+            inverse non-linear transform from the EPI space to the anatomical space
         resampled_bold
             Original BOLD timeseries resampled through motion realignment and
             susceptibility distortion correction based on registration to the
@@ -114,30 +110,39 @@ def init_bold_main_wf(opts, bias_cor_only=False, name='bold_main_wf'):
             EPI WM mask for commonspace bold
         commonspace_CSF_mask
             EPI CSF mask for commonspace bold
+        commonspace_vascular_mask
+            EPI vascular mask for commonspace bold
         commonspace_labels
             EPI anatomical labels for commonspace bold
     """
 
     workflow = pe.Workflow(name=name)
 
-    inputnode = pe.Node(niu.IdentityInterface(fields=['subject_id', 'bold', 'anat_ref', 'anat_mask', 'WM_mask', 'CSF_mask', 'vascular_mask', 'labels', 'template_to_common_affine', 'template_to_common_warp', 'anat_to_template_affine', 'anat_to_template_warp', 'template_anat']),
+    inputnode = pe.Node(niu.IdentityInterface(
+                fields=['bold', 'inho_cor_anat', 'inho_cor_mask', 'coreg_anat', 'coreg_mask',
+                        'native_to_commonspace_transform_list','native_to_commonspace_inverse_list',
+                        'commonspace_to_native_transform_list','commonspace_to_native_inverse_list',
+                        'commonspace_ref']),
                         name="inputnode")
 
     outputnode = pe.Node(niu.IdentityInterface(
-                fields=['input_bold', 'bold_ref', 'motcorr_params', 'init_denoise', 'denoise_mask', 'corrected_EPI', 'output_warped_bold', 'affine_bold2anat', 'warp_bold2anat', 'inverse_warp_bold2anat', 'resampled_bold', 'resampled_ref_bold', 'EPI_brain_mask', 'EPI_WM_mask', 'EPI_CSF_mask', 'EPI_labels',
-                        'confounds_csv', 'FD_voxelwise', 'pos_voxelwise', 'FD_csv', 'commonspace_bold', 'commonspace_mask', 'commonspace_WM_mask', 'commonspace_CSF_mask', 'commonspace_vascular_mask', 'commonspace_labels']),
+                fields=['input_bold', 'bold_ref', 'motcorr_params', 'init_denoise', 'denoise_mask', 'corrected_EPI',
+                        'output_warped_bold', 'bold_to_anat_affine', 'bold_to_anat_warp', 'bold_to_anat_inverse_warp',
+                        'native_bold', 'native_bold_ref', 'native_brain_mask', 'native_WM_mask', 'native_CSF_mask', 'native_labels',
+                        'confounds_csv', 'FD_voxelwise', 'pos_voxelwise', 'FD_csv', 'commonspace_bold', 'commonspace_mask',
+                        'commonspace_WM_mask', 'commonspace_CSF_mask', 'commonspace_vascular_mask', 'commonspace_labels']),
                 name='outputnode')
 
     boldbuffer = pe.Node(niu.IdentityInterface(fields=['bold_file']),
                          name="boldbuffer")
 
-    # this node will serve as a relay of outputs from the bias_cor main_wf to the inputs for the rest of the main_wf for bold_only
+    # this node will serve as a relay of outputs from the inho_cor main_wf to the inputs for the rest of the main_wf for bold_only
     transitionnode = pe.Node(niu.IdentityInterface(fields=['bold_file', 'bold_ref', 'init_denoise', 'denoise_mask', 'corrected_EPI']),
                              name="transitionnode")
 
-    if bias_cor_only or (not opts.bold_only):
+    if inho_cor_only or (not opts.bold_only):
         bold_reference_wf = init_bold_reference_wf(opts=opts)
-        bias_cor_wf = bias_correction_wf(opts=opts)
+        inho_cor_wf = init_inho_correction_wf(opts=opts, image_type='EPI', name="bold_inho_cor_wf")
 
         if opts.apply_despiking:
             despike = pe.Node(
@@ -166,28 +171,28 @@ def init_bold_main_wf(opts, bias_cor_only=False, name='bold_main_wf'):
                 ])
 
         workflow.connect([
-            (inputnode, bias_cor_wf, [
-                ('anat_ref', 'inputnode.anat'),
-                ('anat_mask', 'inputnode.anat_mask'),
+            (inputnode, inho_cor_wf, [
+                ('inho_cor_anat', 'inputnode.anat_ref'),
+                ('inho_cor_mask', 'inputnode.anat_mask'),
                 ('bold', 'inputnode.name_source'),
                 ]),
             (boldbuffer, bold_reference_wf, [
                 ('bold_file', 'inputnode.bold_file'),
                 ]),
-            (bold_reference_wf, bias_cor_wf, [
-                ('outputnode.ref_image', 'inputnode.ref_EPI'),
+            (bold_reference_wf, inho_cor_wf, [
+                ('outputnode.ref_image', 'inputnode.target_img'),
                 ]),
             (bold_reference_wf, transitionnode, [
                 ('outputnode.ref_image', 'bold_ref'),
                 ]),
-            (bias_cor_wf, transitionnode, [
+            (inho_cor_wf, transitionnode, [
                 ('outputnode.init_denoise', 'init_denoise'),
                 ('outputnode.denoise_mask', 'denoise_mask'),
-                ('outputnode.corrected_EPI', 'corrected_EPI'),
+                ('outputnode.corrected', 'corrected_EPI'),
                 ]),
             ])
 
-    if opts.bold_only and bias_cor_only:
+    if inho_cor_only:
         return workflow
 
     bold_stc_wf = init_bold_stc_wf(opts=opts)
@@ -195,44 +200,111 @@ def init_bold_main_wf(opts, bias_cor_only=False, name='bold_main_wf'):
     # HMC on the BOLD
     bold_hmc_wf = init_bold_hmc_wf(opts=opts)
 
-    if not opts.bold_only:
-        def commonspace_transforms(template_to_common_warp, template_to_common_affine, anat_to_template_warp, anat_to_template_affine, warp_bold2anat, affine_bold2anat):
-            # transforms_list,inverses
-            return [template_to_common_warp, template_to_common_affine, anat_to_template_warp, anat_to_template_affine, warp_bold2anat, affine_bold2anat], [0, 0, 0, 0, 0, 0]
-        commonspace_transforms_prep = pe.Node(Function(input_names=['template_to_common_warp', 'template_to_common_affine', 'anat_to_template_warp', 'anat_to_template_affine', 'warp_bold2anat', 'affine_bold2anat'],
-                                                       output_names=[
-                                                           'transforms_list', 'inverses'],
-                                                       function=commonspace_transforms),
-                                              name='commonspace_transforms_prep')
-    else:
-        def commonspace_transforms(template_to_common_warp, template_to_common_affine, anat_to_template_warp, anat_to_template_affine):
-            # transforms_list,inverses
-            return [template_to_common_warp, template_to_common_affine, anat_to_template_warp, anat_to_template_affine], [0, 0, 0, 0]
-        commonspace_transforms_prep = pe.Node(Function(input_names=['template_to_common_warp', 'template_to_common_affine', 'anat_to_template_warp', 'anat_to_template_affine', ],
-                                                       output_names=[
-                                                           'transforms_list', 'inverses'],
-                                                       function=commonspace_transforms),
-                                              name='commonspace_transforms_prep')
-
-    bold_commonspace_trans_wf = init_bold_commonspace_trans_wf(opts=opts)
+    bold_commonspace_trans_wf = init_bold_preproc_trans_wf(opts=opts, resampling_dim=opts.commonspace_resampling, name='bold_commonspace_trans_wf')
+    bold_commonspace_trans_wf.inputs.inputnode.mask_transforms_list = []
+    bold_commonspace_trans_wf.inputs.inputnode.mask_inverses = []
 
     bold_confs_wf = init_bold_confs_wf(opts=opts, name="bold_confs_wf")
 
+    if not opts.bold_only:
+        def commonspace_transforms(to_commonspace_transform_list,to_commonspace_inverse_list, bold_to_anat_warp, bold_to_anat_affine):
+            # transforms_list,inverses
+            return to_commonspace_transform_list+[bold_to_anat_warp, bold_to_anat_affine], to_commonspace_inverse_list+[0,0]
+        bold_to_commonspace_transforms = pe.Node(Function(input_names=['to_commonspace_transform_list','to_commonspace_inverse_list', 'bold_to_anat_warp', 'bold_to_anat_affine'],
+                                                       output_names=[
+                                                           'to_commonspace_transform_list','to_commonspace_inverse_list'],
+                                                       function=commonspace_transforms),
+                                              name='bold_to_commonspace_transforms')
+
+        cross_modal_reg_wf = init_cross_modal_reg_wf(opts=opts)
+
+        def SyN_coreg_transforms_prep(bold_to_anat_warp, bold_to_anat_affine):
+            # transforms_list,inverses
+            return [bold_to_anat_warp, bold_to_anat_affine], [0, 0]
+        transforms_prep = pe.Node(Function(input_names=['bold_to_anat_warp', 'bold_to_anat_affine'],
+                                           output_names=[
+                                               'transforms_list', 'inverses'],
+                                           function=SyN_coreg_transforms_prep),
+                                  name='transforms_prep')
+
+        bold_native_trans_wf = init_bold_preproc_trans_wf(opts=opts, resampling_dim=opts.nativespace_resampling, name='bold_native_trans_wf')
+
+        workflow.connect([
+            (inputnode, cross_modal_reg_wf, [
+                ('coreg_anat', 'inputnode.anat_ref'),
+                ('coreg_mask', 'inputnode.anat_mask')]),
+            (inputnode, bold_native_trans_wf, [
+                ('commonspace_to_native_transform_list', 'inputnode.mask_transforms_list'),
+                ('commonspace_to_native_inverse_list', 'inputnode.mask_inverses'),
+                ('bold', 'inputnode.name_source'),
+                ]),
+            (transitionnode, cross_modal_reg_wf, [
+                ('corrected_EPI', 'inputnode.ref_bold_brain'),
+                ('denoise_mask', 'inputnode.moving_mask'),
+                ]),
+            (cross_modal_reg_wf, outputnode, [
+                ('outputnode.bold_to_anat_affine', 'bold_to_anat_affine'),
+                ('outputnode.bold_to_anat_warp', 'bold_to_anat_warp'),
+                ('outputnode.bold_to_anat_inverse_warp', 'bold_to_anat_inverse_warp'),
+                ('outputnode.output_warped_bold', 'output_warped_bold'),
+                ]),
+            (cross_modal_reg_wf, transforms_prep, [
+                ('outputnode.bold_to_anat_affine', 'bold_to_anat_affine'),
+                ('outputnode.bold_to_anat_warp', 'bold_to_anat_warp'),
+                ]),
+            (transforms_prep, bold_native_trans_wf, [
+                ('transforms_list', 'inputnode.transforms_list'),
+                ('inverses', 'inputnode.inverses'),
+                ]),
+            (cross_modal_reg_wf, bold_native_trans_wf, [
+                ('outputnode.output_warped_bold', 'inputnode.ref_file')]),
+            (cross_modal_reg_wf, bold_to_commonspace_transforms, [
+                ('outputnode.bold_to_anat_affine', 'bold_to_anat_affine'),
+                ('outputnode.bold_to_anat_warp', 'bold_to_anat_warp'),
+                ]),
+            (bold_hmc_wf, bold_native_trans_wf, [
+             ('outputnode.motcorr_params', 'inputnode.motcorr_params')]),
+            (bold_native_trans_wf, bold_confs_wf, [
+                ('outputnode.bold', 'inputnode.bold'),
+                ('outputnode.bold_ref','inputnode.ref_bold'),
+                ('outputnode.brain_mask', 'inputnode.brain_mask'),
+                ('outputnode.WM_mask', 'inputnode.WM_mask'),
+                ('outputnode.CSF_mask', 'inputnode.CSF_mask'),
+                ('outputnode.vascular_mask', 'inputnode.vascular_mask'),
+                ]),
+            (bold_native_trans_wf, outputnode, [
+                ('outputnode.bold', 'native_bold'),
+                ('outputnode.bold_ref','native_bold_ref'),
+                ('outputnode.brain_mask', 'native_brain_mask'),
+                ('outputnode.WM_mask', 'native_WM_mask'),
+                ('outputnode.CSF_mask', 'native_CSF_mask'),
+                ('outputnode.vascular_mask', 'native_vascular_mask'),
+                ('outputnode.labels', 'native_labels'),
+                ]),
+            ])
+
+    else:
+        bold_to_commonspace_transforms = pe.Node(niu.IdentityInterface(fields=['to_commonspace_transform_list','to_commonspace_inverse_list']),
+                                 name="bold_to_commonspace_transforms")
+
+        workflow.connect([
+            (bold_commonspace_trans_wf, bold_confs_wf, [
+                ('outputnode.bold', 'inputnode.bold'),
+                ('outputnode.bold_ref','inputnode.ref_bold'),
+                ('outputnode.brain_mask', 'inputnode.brain_mask'),
+                ('outputnode.WM_mask', 'inputnode.WM_mask'),
+                ('outputnode.CSF_mask', 'inputnode.CSF_mask'),
+                ('outputnode.vascular_mask', 'inputnode.vascular_mask'),
+                ]),
+            ])
+
+
     # MAIN WORKFLOW STRUCTURE #######################################################
     workflow.connect([
-        (inputnode, commonspace_transforms_prep, [
-            ("template_to_common_affine", "template_to_common_affine"),
-            ("template_to_common_warp", "template_to_common_warp"),
-            ("anat_to_template_affine", "anat_to_template_affine"),
-            ("anat_to_template_warp", "anat_to_template_warp"),
+        (inputnode, bold_to_commonspace_transforms, [
+            ('native_to_commonspace_transform_list', 'to_commonspace_transform_list'),
+            ('native_to_commonspace_inverse_list', 'to_commonspace_inverse_list'),
             ]),
-        (inputnode, bold_confs_wf, [('anat_mask', 'inputnode.t1_mask'),
-                                    ('WM_mask', 'inputnode.WM_mask'),
-                                    ('CSF_mask', 'inputnode.CSF_mask'),
-                                    ('vascular_mask', 'inputnode.vascular_mask'),
-                                    ('labels', 'inputnode.t1_labels'),
-                                    ('bold', 'inputnode.name_source'),
-                                    ]),
         (transitionnode, bold_stc_wf, [
             ('bold_file', 'inputnode.bold_file'),
             ]),
@@ -251,24 +323,20 @@ def init_bold_main_wf(opts, bias_cor_only=False, name='bold_main_wf'):
             ('outputnode.motcorr_params', 'inputnode.movpar_file'),
             ]),
         (bold_confs_wf, outputnode, [
-            ('outputnode.brain_mask', 'EPI_brain_mask'),
-            ('outputnode.WM_mask', 'EPI_WM_mask'),
-            ('outputnode.CSF_mask', 'EPI_CSF_mask'),
-            ('outputnode.EPI_labels', 'EPI_labels'),
             ('outputnode.confounds_csv', 'confounds_csv'),
             ('outputnode.FD_csv', 'FD_csv'),
             ('outputnode.FD_voxelwise', 'FD_voxelwise'),
             ('outputnode.pos_voxelwise', 'pos_voxelwise'),
             ]),
-        (commonspace_transforms_prep, bold_commonspace_trans_wf, [
-            ('transforms_list', 'inputnode.transforms_list'),
-            ('inverses', 'inputnode.inverses'),
+        (bold_to_commonspace_transforms, bold_commonspace_trans_wf, [
+            ('to_commonspace_transform_list', 'inputnode.transforms_list'),
+            ('to_commonspace_inverse_list', 'inputnode.inverses'),
             ]),
         (bold_hmc_wf, bold_commonspace_trans_wf, [
          ('outputnode.motcorr_params', 'inputnode.motcorr_params')]),
         (inputnode, bold_commonspace_trans_wf, [
             ('bold', 'inputnode.name_source'),
-            ('template_anat', 'inputnode.ref_file'),
+            ('commonspace_ref', 'inputnode.ref_file'),
             ]),
         (bold_commonspace_trans_wf, outputnode, [
             ('outputnode.bold', 'commonspace_bold'),
@@ -280,68 +348,6 @@ def init_bold_main_wf(opts, bias_cor_only=False, name='bold_main_wf'):
             ]),
         ])
 
-    if not opts.bold_only:
-        bold_reg_wf = init_bold_reg_wf(opts=opts)
-
-        def SyN_coreg_transforms_prep(warp_bold2anat, affine_bold2anat):
-            # transforms_list,inverses
-            return [warp_bold2anat, affine_bold2anat], [0, 0]
-        transforms_prep = pe.Node(Function(input_names=['warp_bold2anat', 'affine_bold2anat'],
-                                           output_names=[
-                                               'transforms_list', 'inverses'],
-                                           function=SyN_coreg_transforms_prep),
-                                  name='transforms_prep')
-
-        # Apply transforms in 1 shot
-        bold_bold_trans_wf = init_bold_preproc_trans_wf(opts=opts)
-
-        workflow.connect([
-            (inputnode, bold_reg_wf, [
-                ('anat_ref', 'inputnode.anat_ref'),
-                ('anat_mask', 'inputnode.anat_mask')]),
-            (inputnode, bold_bold_trans_wf, [
-                ('bold', 'inputnode.name_source')]),
-            (transitionnode, bold_reg_wf, [
-                ('corrected_EPI', 'inputnode.ref_bold_brain')]),
-            (bold_reg_wf, outputnode, [
-                ('outputnode.affine_bold2anat', 'affine_bold2anat'),
-                ('outputnode.warp_bold2anat', 'warp_bold2anat'),
-                ('outputnode.inverse_warp_bold2anat', 'inverse_warp_bold2anat'),
-                ('outputnode.output_warped_bold', 'output_warped_bold'),
-                ]),
-            (bold_reg_wf, transforms_prep, [
-                ('outputnode.affine_bold2anat', 'affine_bold2anat'),
-                ('outputnode.warp_bold2anat', 'warp_bold2anat'),
-                ]),
-            (transforms_prep, bold_bold_trans_wf, [
-                ('transforms_list', 'inputnode.transforms_list'),
-                ('inverses', 'inputnode.inverses'),
-                ]),
-            (bold_reg_wf, bold_bold_trans_wf, [
-                ('outputnode.output_warped_bold', 'inputnode.ref_file')]),
-            (bold_reg_wf, commonspace_transforms_prep, [
-                ('outputnode.affine_bold2anat', 'affine_bold2anat'),
-                ('outputnode.warp_bold2anat', 'warp_bold2anat'),
-                ]),
-            (bold_hmc_wf, bold_bold_trans_wf, [
-             ('outputnode.motcorr_params', 'inputnode.motcorr_params')]),
-            (bold_bold_trans_wf, outputnode, [
-                ('outputnode.bold_ref', 'resampled_ref_bold'),
-                ('outputnode.bold', 'resampled_bold'),
-                ]),
-            (bold_bold_trans_wf, bold_confs_wf, [('outputnode.bold', 'inputnode.bold'),
-                                                 ('outputnode.bold_ref',
-                                                  'inputnode.ref_bold'),
-                                                 ]),
-            ])
-    else:
-        workflow.connect([
-            (bold_commonspace_trans_wf, bold_confs_wf, [('outputnode.bold', 'inputnode.bold'),
-                                                        ('outputnode.bold_ref',
-                                                         'inputnode.ref_bold'),
-                                                        ]),
-            ])
-
     if opts.apply_slice_mc:
         workflow.connect([
             (bold_stc_wf, bold_hmc_wf, [
@@ -351,7 +357,7 @@ def init_bold_main_wf(opts, bias_cor_only=False, name='bold_main_wf'):
         ])
         if not opts.bold_only:
             workflow.connect([
-                (bold_hmc_wf, bold_bold_trans_wf, [
+                (bold_hmc_wf, bold_native_trans_wf, [
                  ('outputnode.slice_corrected_bold', 'inputnode.bold_file')]),
             ])
     else:
@@ -363,7 +369,7 @@ def init_bold_main_wf(opts, bias_cor_only=False, name='bold_main_wf'):
         ])
         if not opts.bold_only:
             workflow.connect([
-                (bold_stc_wf, bold_bold_trans_wf, [
+                (bold_stc_wf, bold_native_trans_wf, [
                  ('outputnode.stc_file', 'inputnode.bold_file')]),
             ])
 
